# P2-HOST-FS
HOST FS - code for accessing the host filesystem via the 9P protocol

The following files can be found at the top level of this repository:

| Key Files | Description |
| --- | --- |
| [`hostfs.spin2`](./hostfs.spin2) | The complete FLASH driver object
| [`hostfs_doc.txt`](./hostfs_doc.txt) | Object Interface Document for the HostFs driver
| [`host_file_demo.spin2`](./host_file_demo.spin2) | Interactive Filesystem Demo based on code from Jon McPhalen
| [`demo.spin2`](./demo.spin2) | Simpler demo of the file system

## Host Filesystem Features

Key features of this filesystem for the P2 chip:

- Read and write files and directories directly on the host PC
- Transport agnostic; defaults to using the serial port, but can easily be modified to use ethernet or other connections
- Uses the 9P protocol, which is a widely used and simple protocol for network file systems


## Adding the Host FS to your own project

This section describes how to quickly get the host filesystem working in your project.

### Include the Host FS object in your top file

Place the downloaded `hostfs.spin2` in your project and in your top-level object include the flash object:

```spin2
OBJ { Objects Used by this Object }

    host  : "hostfs"	           ' the host filesystem
```

### Provide a Serial (or other) Interface for the File System

Hostfs needs to talk to the host by some mechanism. The actual interface is very simple: it needs to be able to send a block of data and receive another block as response. The API for this is:
```
pub sendrecv(bufstart, bufend, maxlen) : reply_len
```
where `bufstart` is a pointer to the start of the data to send (and is also where the received data will be placed), `bufend` is a pointer to the end of the data to send, and `maxlen` is the maximum size of the buffer. The function returns the actual number of bytes received in the reply from the host.

#### Serial connection using existing serial object

The most straightforward connection is the serial port, preferably at a very high baud rate (like 2_000_000 baud). If you already have a serial object that you prefer to use (e.g. Jon McPhalen's excellent jm_fullduplexserial.spin2) then you can use the `serial_sendrecv` object to create a `sendrecv` function that will work with hostfs. The general usage is:
```
obj
  host : "hostfs"
  term : "jm_fullduplexserial"  ' or any other serial object
  fssr : "serial_sendrecv"      ' interface
...
  term.start(BAUD)
  status := host.mount(fssr.init(@term.tx, @term.rx))
```
The `fssr` object, an instance of "serial_sendrecv", has only one method, `init`, which takes as parameters pointers to single byte transmit and single byte receive methods, and returns a method pointer suitable for passing to the hostfs `mount` method.

#### Serial connection using serial_9p object

If you don't already have a favorite serial object, a simple serial object with built in sendrecv method is provided as "serial_9p.spin2". Using this is very easy:
```
obj
  host : "hostfs"
  term : "serial_9p"
...
  term.start(BAUD)
  status := host.mount(@term.sendrecv)
```

#### Other connections

To use the host file system with a different interface (e.g. ethernet), you must pass a sendrecv() method pointer to the file system `mount` procedure. sendrecv() looks like:
```
pub sendrecv(startptr, endptr, maxrecvlen) : bytes_recv
```
where:

  - `startptr` is the start of the read/write buffer
  - `endptr`   is the end of the data to send (so total bytes to send is `endptr - startptr`
  - `maxrecvlen` is the size of the buffer starting at `startptr`. This should generally be of size `host.FS_MAXLEN` bytes (currently 1048 bytes).


### Mount() the host filesystem

At the start of your application will need to mount() the filesystem. The mount() function takes as a parameter a pointer to a `sendrecv` method that looks like

```spin2
PRI main() | status, sendrecvf
    'sendrecvf := @ser.sendrecv                    ' if your serial object already has a sendrecv method
    sendrecvf := sendrecvfs.init(@ser.tx, @ser.rx) ' if ser does not already have sendrecv 
    status := host.mount(sendrecvf)	           ' start up the FLASH filesystem
    if status <> host.SUCCESS
      '... you know that some blocks were corrupted and freed ...
```

### Start a file server on the PC side

The serial file server is built in to the FlexProp GUI, but also available in the command line loadp2 and proploader tools. Both use the `-9<dir>` option to specify that the serial file server should be started with `<dir>` as the root. For example, `-9.` starts the serial file server using the current directory as the root, or `-9Test/` starts the serial file server using subdirectory `Test` as the root.

Example of running program demo.binary at 2000000 baud with file server serving files from the current directory:
```
loadp2 -b2000000 demo.binary -9.
```

For ethernet or other connections, you'll have to start your own 9P server. Many examples are available on-line.

### Work with files as you would normally

Now that your filesystem is mounted you are free to do normal file operations. The filesystem API is very similar to the ANSI-C file handling API so this all should feel pretty natural to you.


## Credits

Thank you to **Stephen Moraco** and **Chip Gracey** for the excellent **Flash File System** written for the P2 which heavily inspired the interface for this file system. 

Thank you to **Jon McPhalen** for his serial port code and file system demo. I have modified his demo to use the Host file system rather than Flash; in the process I have probably made his very clean code a little less clean, and that is entirely my fault, not his!


---

## Disclaimer and Legal

> *Parallax, Propeller Spin, and the Parallax and Propeller Hat logos* are trademarks of Parallax Inc., dba Parallax Semiconductor
>
> This project is in no way affiliated with, authorized, maintained, sponsored or endorsed by *Parallax Inc., dba Parallax Semiconductor* or any of its affiliates or subsidiaries.

---

## License

Licensed under the MIT License. Copyright 2023 Total Spectrum Software Inc.

